@@ -1,26 +1,19 @@
 
 # src/codegraphcontext/tools/graph_builder.py
 import asyncio
-import logging
+import pathspec
 from pathlib import Path
 from typing import Any, Coroutine, Dict, Optional, Tuple
 from datetime import datetime
 
 from ..core.database import DatabaseManager
 from ..core.jobs import JobManager, JobStatus
-from ..utils.debug_log import debug_log
+from ..utils.debug_log import debug_log, info_logger, error_logger, warning_logger
 
 # New imports for tree-sitter
 from tree_sitter import Language, Parser
 from tree_sitter_languages import get_language
 
-logger = logging.getLogger(__name__)
-
-# This is for developers and testers only. It enables detailed debug logging to a file.
-# Set to 1 to enable, 0 to disable.
-debug_mode = 0
-
-
 class TreeSitterParser:
     """A generic parser wrapper for a specific language using tree-sitter."""
 
@@ -58,6 +51,9 @@ class TreeSitterParser:
         elif self.language_name == 'ruby':
             from .languages.ruby import RubyTreeSitterParser
             self.language_specific_parser = RubyTreeSitterParser(self)
+        elif self.language_name == 'c_sharp':
+            from .languages.csharp import CSharpTreeSitterParser
+            self.language_specific_parser = CSharpTreeSitterParser(self)
 
 
     def parse(self, file_path: Path, is_dependency: bool = False, **kwargs) -> Dict:
@@ -92,7 +88,8 @@ class GraphBuilder:
             '.c': TreeSitterParser('c'),
             # '.h': TreeSitterParser('c'), # Need to write an algo for distinguishing C vs C++ headers
             '.java': TreeSitterParser('java'),
-            '.rb': TreeSitterParser('ruby')
+            '.rb': TreeSitterParser('ruby'),
+            '.cs': TreeSitterParser('c_sharp')
         }
         self.create_schema()
 
@@ -107,24 +104,31 @@ class GraphBuilder:
                 session.run("CREATE CONSTRAINT directory_path IF NOT EXISTS FOR (d:Directory) REQUIRE d.path IS UNIQUE")
                 session.run("CREATE CONSTRAINT function_unique IF NOT EXISTS FOR (f:Function) REQUIRE (f.name, f.file_path, f.line_number) IS UNIQUE")
                 session.run("CREATE CONSTRAINT class_unique IF NOT EXISTS FOR (c:Class) REQUIRE (c.name, c.file_path, c.line_number) IS UNIQUE")
+                session.run("CREATE CONSTRAINT trait_unique IF NOT EXISTS FOR (t:Trait) REQUIRE (t.name, t.file_path, t.line_number) IS UNIQUE") # Added trait constraint
                 session.run("CREATE CONSTRAINT interface_unique IF NOT EXISTS FOR (i:Interface) REQUIRE (i.name, i.file_path, i.line_number) IS UNIQUE")
                 session.run("CREATE CONSTRAINT macro_unique IF NOT EXISTS FOR (m:Macro) REQUIRE (m.name, m.file_path, m.line_number) IS UNIQUE")
                 session.run("CREATE CONSTRAINT variable_unique IF NOT EXISTS FOR (v:Variable) REQUIRE (v.name, v.file_path, v.line_number) IS UNIQUE")
                 session.run("CREATE CONSTRAINT module_name IF NOT EXISTS FOR (m:Module) REQUIRE m.name IS UNIQUE")
-
+                session.run("CREATE CONSTRAINT struct_cpp IF NOT EXISTS FOR (cstruct: Struct) REQUIRE (cstruct.name, cstruct.file_path, cstruct.line_number) IS UNIQUE")
+                session.run("CREATE CONSTRAINT enum_cpp IF NOT EXISTS FOR (cenum: Enum) REQUIRE (cenum.name, cenum.file_path, cenum.line_number) IS UNIQUE")
+                session.run("CREATE CONSTRAINT union_cpp IF NOT EXISTS FOR (cunion: Union) REQUIRE (cunion.name, cunion.file_path, cunion.line_number) IS UNIQUE")
+                session.run("CREATE CONSTRAINT annotation_unique IF NOT EXISTS FOR (a:Annotation) REQUIRE (a.name, a.file_path, a.line_number) IS UNIQUE")
+                session.run("CREATE CONSTRAINT record_unique IF NOT EXISTS FOR (r:Record) REQUIRE (r.name, r.file_path, r.line_number) IS UNIQUE")
+                session.run("CREATE CONSTRAINT property_unique IF NOT EXISTS FOR (p:Property) REQUIRE (p.name, p.file_path, p.line_number) IS UNIQUE")
+                
                 # Indexes for language attribute
                 session.run("CREATE INDEX function_lang IF NOT EXISTS FOR (f:Function) ON (f.lang)")
                 session.run("CREATE INDEX class_lang IF NOT EXISTS FOR (c:Class) ON (c.lang)")
-                
+                session.run("CREATE INDEX annotation_lang IF NOT EXISTS FOR (a:Annotation) ON (a.lang)")
                 session.run("""
                     CREATE FULLTEXT INDEX code_search_index IF NOT EXISTS 
                     FOR (n:Function|Class|Variable) 
                     ON EACH [n.name, n.source, n.docstring]
                 """ )
                 
-                logger.info("Database schema verified/created successfully")
+                info_logger("Database schema verified/created successfully")
             except Exception as e:
-                logger.warning(f"Schema creation warning: {e}")
+                warning_logger(f"Schema creation warning: {e}")
 
 
     def _pre_scan_for_imports(self, files: list[Path]) -> dict:
@@ -188,6 +192,9 @@ class GraphBuilder:
         elif '.rb' in files_by_lang:
             from .languages import ruby as ruby_lang_module
             imports_map.update(ruby_lang_module.pre_scan_ruby(files_by_lang['.rb'], self.parsers['.rb']))
+        elif '.cs' in files_by_lang:
+            from .languages import csharp as csharp_lang_module
+            imports_map.update(csharp_lang_module.pre_scan_csharp(files_by_lang['.cs'], self.parsers['.cs']))
             
         return imports_map
 
@@ -209,7 +216,7 @@ class GraphBuilder:
 
     # First pass to add file and its contents
     def add_file_to_graph(self, file_data: Dict, repo_name: str, imports_map: dict):
-        logger.info("Executing add_file_to_graph with my change!")
+        info_logger("Executing add_file_to_graph with my change!")
         """Adds a file and its contents within a single, unified session."""
         file_path_str = str(Path(file_data['file_path']).resolve())
         file_name = Path(file_path_str).name
@@ -264,9 +271,15 @@ class GraphBuilder:
             item_mappings = [
                 (file_data.get('functions', []), 'Function'),
                 (file_data.get('classes', []), 'Class'),
+                (file_data.get('traits', []), 'Trait'), # <-- Added trait mapping
                 (file_data.get('variables', []), 'Variable'),
                 (file_data.get('interfaces', []), 'Interface'),
-                (file_data.get('macros', []), 'Macro')
+                (file_data.get('macros', []), 'Macro'),
+                (file_data.get('structs',[]), 'Struct'),
+                (file_data.get('enums',[]), 'Enum'),
+                (file_data.get('unions',[]), 'Union'),
+                (file_data.get('records',[]), 'Record'),
+                (file_data.get('properties',[]), 'Property'),
             ]
             for item_data, label in item_mappings:
                 for item in item_data:
@@ -290,6 +303,14 @@ class GraphBuilder:
                                 MERGE (fn)-[:HAS_PARAMETER]->(p)
                             """, func_name=item['name'], file_path=file_path_str, line_number=item['line_number'], arg_name=arg_name)
 
+            # --- NEW: persist Ruby Modules ---
+            for m in file_data.get('modules', []):
+                session.run("""
+                    MERGE (mod:Module {name: $name})
+                    ON CREATE SET mod.lang = $lang
+                    ON MATCH  SET mod.lang = coalesce(mod.lang, $lang)
+                """, name=m["name"], lang=file_data.get("lang"))
+
             # Create CONTAINS relationships for nested functions
             for item in file_data.get('functions', []):
                 if item.get("context_type") == "function_definition":
@@ -301,7 +322,7 @@ class GraphBuilder:
 
             # Handle imports and create IMPORTS relationships
             for imp in file_data.get('imports', []):
-                logger.info(f"Processing import: {imp}")
+                info_logger(f"Processing import: {imp}")
                 lang = file_data.get('lang')
                 if lang == 'javascript':
                     # New, correct logic for JS
@@ -346,6 +367,17 @@ class GraphBuilder:
                     func_name=func['name'],
                     func_line=func['line_number'])
 
+            # --- NEW: Class INCLUDES Module (Ruby mixins) ---
+            for inc in file_data.get('module_inclusions', []):
+                session.run("""
+                    MATCH (c:Class {name: $class_name, file_path: $file_path})
+                    MERGE (m:Module {name: $module_name})
+                    MERGE (c)-[:INCLUDES]->(m)
+                """,
+                class_name=inc["class"],
+                file_path=file_path_str,
+                module_name=inc["module"])
+
             # Class inheritance is handled in a separate pass after all files are processed.
             # Function calls are also handled in a separate pass after all files are processed.
 
@@ -489,11 +521,82 @@ class GraphBuilder:
                     parent_name=target_class_name,
                     resolved_parent_file_path=resolved_path)
 
+
+    def _create_csharp_inheritance_and_interfaces(self, session, file_data: Dict, imports_map: dict):
+        """Create INHERITS and IMPLEMENTS relationships for C# types."""
+        if file_data.get('lang') != 'c_sharp':
+            return
+            
+        caller_file_path = str(Path(file_data['file_path']).resolve())
+        
+        # Collect all local type names
+        local_type_names = set()
+        for type_list in ['classes', 'interfaces', 'structs', 'records']:
+            local_type_names.update(t['name'] for t in file_data.get(type_list, []))
+        
+        # Process all type declarations that can have bases
+        for type_list_name, type_label in [('classes', 'Class'), ('structs', 'Struct'), ('records', 'Record'), ('interfaces', 'Interface')]:
+            for type_item in file_data.get(type_list_name, []):
+                if not type_item.get('bases'):
+                    continue
+                
+                for base_str in type_item['bases']:
+                    # Clean up the base name (remove generic parameters, etc.)
+                    base_name = base_str.split('<')[0].strip()
+                    
+                    # Determine if this is an interface
+                    is_interface = False
+                    resolved_path = caller_file_path
+                    
+                    # Check if base is a local interface
+                    for iface in file_data.get('interfaces', []):
+                        if iface['name'] == base_name:
+                            is_interface = True
+                            break
+                    
+                    # Check if base is in imports_map
+                    if base_name in imports_map:
+                        possible_paths = imports_map[base_name]
+                        if len(possible_paths) > 0:
+                            resolved_path = possible_paths[0]
+                    
+                    # For C#, first base is usually the class (if any), rest are interfaces
+                    base_index = type_item['bases'].index(base_str)
+                    
+                    # Try to determine if it's an interface
+                    if is_interface or (base_index > 0 and type_label == 'Class'):
+                        # This is an IMPLEMENTS relationship
+                        session.run("""
+                            MATCH (child {name: $child_name, file_path: $file_path})
+                            WHERE child:Class OR child:Struct OR child:Record
+                            MATCH (iface:Interface {name: $interface_name})
+                            MERGE (child)-[:IMPLEMENTS]->(iface)
+                        """,
+                        child_name=type_item['name'],
+                        file_path=caller_file_path,
+                        interface_name=base_name)
+                    else:
+                        # This is an INHERITS relationship
+                        session.run("""
+                            MATCH (child {name: $child_name, file_path: $file_path})
+                            WHERE child:Class OR child:Record OR child:Interface
+                            MATCH (parent {name: $parent_name})
+                            WHERE parent:Class OR parent:Record OR parent:Interface
+                            MERGE (child)-[:INHERITS]->(parent)
+                        """,
+                        child_name=type_item['name'],
+                        file_path=caller_file_path,
+                        parent_name=base_name)
+
     def _create_all_inheritance_links(self, all_file_data: list[Dict], imports_map: dict):
         """Create INHERITS relationships for all classes after all files have been processed."""
         with self.driver.session() as session:
             for file_data in all_file_data:
-                self._create_inheritance_links(session, file_data, imports_map)
+                # Handle C# separately
+                if file_data.get('lang') == 'c_sharp':
+                    self._create_csharp_inheritance_and_interfaces(session, file_data, imports_map)
+                else:
+                    self._create_inheritance_links(session, file_data, imports_map)
                 
     def delete_file_from_graph(self, file_path: str):
         """Deletes a file and all its contained elements and relationships."""
@@ -513,7 +616,7 @@ class GraphBuilder:
                 """,
                 path=file_path_str,
             )
-            logger.info(f"Deleted file and its elements from graph: {file_path_str}")
+            info_logger(f"Deleted file and its elements from graph: {file_path_str}")
 
             for path in parent_paths:
                 session.run("""
@@ -529,7 +632,7 @@ class GraphBuilder:
             session.run("""MATCH (r:Repository {path: $path})
                           OPTIONAL MATCH (r)-[:CONTAINS*]->(e)
                           DETACH DELETE r, e""", path=repo_path_str)
-            logger.info(f"Deleted repository and its contents from graph: {repo_path_str}")
+            info_logger(f"Deleted repository and its contents from graph: {repo_path_str}")
 
     def update_file_in_graph(self, file_path: Path, repo_path: Path, imports_map: dict):
         """Updates a single file's nodes in the graph."""
@@ -545,7 +648,7 @@ class GraphBuilder:
                 self.add_file_to_graph(file_data, repo_name, imports_map)
                 return file_data
             else:
-                logger.error(f"Skipping graph add for {file_path_str} due to parsing error: {file_data['error']}")
+                error_logger(f"Skipping graph add for {file_path_str} due to parsing error: {file_data['error']}")
                 return None
         else:
             return {"deleted": True, "path": file_path_str}
@@ -554,7 +657,7 @@ class GraphBuilder:
         """Parses a file with the appropriate language parser and extracts code elements."""
         parser = self.parsers.get(file_path.suffix)
         if not parser:
-            logger.warning(f"No parser found for file extension {file_path.suffix}. Skipping {file_path}")
+            warning_logger(f"No parser found for file extension {file_path.suffix}. Skipping {file_path}")
             return {"file_path": str(file_path), "error": f"No parser for {file_path.suffix}"}
 
         debug_log(f"[parse_file] Starting parsing for: {file_path} with {parser.language_name} parser")
@@ -565,11 +668,11 @@ class GraphBuilder:
             else:
                 file_data = parser.parse(file_path, is_dependency)
             file_data['repo_path'] = str(repo_path)
-            if debug_mode:
-                debug_log(f"[parse_file] Successfully parsed: {file_path}")
+            debug_log(f"[parse_file] Successfully parsed: {file_path}")
             return file_data
+            
         except Exception as e:
-            logger.error(f"Error parsing {file_path} with {parser.language_name} parser: {e}")
+            error_logger(f"Error parsing {file_path} with {parser.language_name} parser: {e}")
             debug_log(f"[parse_file] Error parsing {file_path}: {e}")
             return {"file_path": str(file_path), "error": str(e)}
 
@@ -590,7 +693,7 @@ class GraphBuilder:
             estimated_time = total_files * 0.05 # tree-sitter is faster
             return total_files, estimated_time
         except Exception as e:
-            logger.error(f"Could not estimate processing time for {path}: {e}")
+            error_logger(f"Could not estimate processing time for {path}: {e}")
             return None
 
     async def build_graph_from_path_async(
@@ -604,9 +707,19 @@ class GraphBuilder:
             self.add_repository_to_graph(path, is_dependency)
             repo_name = path.name
 
+            cgcignore_path = path / ".cgcignore"
+            if cgcignore_path.exists():
+                with open(cgcignore_path) as f:
+                    ignore_patterns = f.read().splitlines()
+                spec = pathspec.PathSpec.from_lines('gitwildmatch', ignore_patterns)
+            else:
+                spec = None
+
             supported_extensions = self.parsers.keys()
             all_files = path.rglob("*") if path.is_dir() else [path]
             files = [f for f in all_files if f.is_file() and f.suffix in supported_extensions]
+            if spec:
+                files = [f for f in files if not spec.match_file(str(f.relative_to(path)))]
             if job_id:
                 self.job_manager.update_job(job_id, total_files=len(files))
             
@@ -638,7 +751,7 @@ class GraphBuilder:
                 self.job_manager.update_job(job_id, status=JobStatus.COMPLETED, end_time=datetime.now())
         except Exception as e:
             error_message=str(e)
-            logger.error(f"Failed to build graph for path {path}: {error_message}", exc_info=True)
+            error_logger(f"Failed to build graph for path {path}: {error_message}")
             if job_id:
                 '''checking if the repo got deleted '''
                 if "no such file found" in error_message or "deleted" in error_message or "not found" in error_message:
